﻿using System;
using System.IO;
using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;

namespace Engine
{
    public class ColorShaderProgram
    {
        protected int _program;
        private int _pixelColorLocation;
        private int _modelTransformLocation;
        private int _viewProjTransformLocation;

        public ColorShaderProgram(
            string vertexShaderPath, string fragmentShaderPath,
            int vertexBuffer)
        {
            _program = -1;

            // Clear a file
            if (File.Exists(Logger.logFileName))
            {
                File.WriteAllText(Logger.logFileName, "");
            }

            string vSource = GetShaderSourceFromFile(vertexShaderPath);
            string fSource = GetShaderSourceFromFile(fragmentShaderPath);
            if (vSource == null || fSource == null)
            {
                return;
            }

            _program = CreateShaderProgram(vSource, fSource);

            GL.BindBuffer(BufferTarget.ArrayBuffer, vertexBuffer);

            // Assign the VBO to aPosition variable
            GL.VertexAttribPointer(0, 2, VertexAttribPointerType.Float, false, 0, 0);
            GL.EnableVertexAttribArray(0);

            _pixelColorLocation = GL.GetUniformLocation(_program, "uPixelColor");
            _modelTransformLocation = GL.GetUniformLocation(_program, "uModelTransform");
            _viewProjTransformLocation = GL.GetUniformLocation(_program, "uViewProjTransform");
            if (_pixelColorLocation < 0 || _modelTransformLocation < 0 || _viewProjTransformLocation < 0)
            {
                Logger.Append("Failed to get an uniform location in ColorShaderProgram.cs");
            }
        }

        public int GetProgramID()
        {
            return _program;
        }

        // Activate the shader for rendering
        public void ActiveShaderProgram(Color4 color, Matrix4 vpMatrix)
        {
            GL.UseProgram(_program);
            GL.UniformMatrix4(_viewProjTransformLocation, false, ref vpMatrix);
            GL.Uniform4(_pixelColorLocation, color);
        }

        public void LoadObjectTransform(Matrix4 modelTransform)
        {
            GL.UniformMatrix4(_modelTransformLocation, false, ref modelTransform);
        }

        private string GetShaderSourceFromFile(string fileName)
        {
            string source = null;

            try
            {
                using (StreamReader sr = new StreamReader(fileName))
                {
                    source = sr.ReadToEnd();
                }
            }
            catch (Exception e)
            {
                Logger.Append(e.Message);
            }

            return source;
        }

        private int CreateShaderProgram(string vSource, string fSource)
        {
            int program = GL.CreateProgram();
            int vShader = CreateShader(ShaderType.VertexShader, vSource);
            int fShader = CreateShader(ShaderType.FragmentShader, fSource);
            if (vShader == -1 || fShader == -1)
            {
                return 0;
            }
            GL.AttachShader(program, vShader);
            GL.AttachShader(program, fShader);
            GL.LinkProgram(program);

            // Check the result of the linking
            int status;
            GL.GetProgram(program, GetProgramParameterName.LinkStatus, out status);
            if (status == 0)
            {
                string errorString = string.Format("Failed to link program: {0}" + Environment.NewLine, GL.GetProgramInfoLog(program));
                Logger.Append(errorString);
                GL.DeleteProgram(program);
                GL.DeleteShader(vShader);
                GL.DeleteShader(fShader);
                return -1;
            }

            return program;
        }

        private int CreateShader(ShaderType shaderType, string source)
        {
            int shader = GL.CreateShader(shaderType);
            GL.ShaderSource(shader, source);
            GL.CompileShader(shader);

            // Check the result of the compilation
            int status;
            GL.GetShader(shader, ShaderParameter.CompileStatus, out status);
            if (status == 0)
            {
                string errorString = string.Format("Failed to compile {0} shader: {1}", shaderType.ToString(), GL.GetShaderInfoLog(shader));
                Logger.Append(errorString);
                GL.DeleteShader(shader);
                return -1;
            }

            return shader;
        }
    }
}