﻿using System;
using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;
using Engine;
using OpenTK.Input;

namespace ColoredAndTexturedObjects
{
    class Window : GameWindow
    {
        private Renderable _colorRect;
        private TextureRenderable _tank;
        private TextureRenderable _bonusStar;
        private TextureRenderable _bonusTank;
        private string _tankTextureName = "Textures/tank_yellow_small_up_01.png";
        private string _bonusStarTextureName = "Textures/bonus_star.png";
        private string _bonusTankTextureName = "Textures/bonus_tank.png";

        private Camera _camera;
        private int _fieldWidth = 10;

        private float _x = 0f;
        private float _y = 0f;
        private float _speed = 0.05f;

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            Width = 400;
            Height = 400;
            Title = "OpenGL 3.1 + OpenTK + C# = :)";

            // Allows transperency with textures
            GL.BlendFunc(BlendingFactorSrc.SrcAlpha, BlendingFactorDest.OneMinusSrcAlpha);
            GL.Enable(EnableCap.Blend);

            Input.GetInstance().Initialize();

            // Get vertex and texture buffers
            VertexBuffer.GetInstance().Initialize();
            int vertexBuffer = VertexBuffer.GetInstance().GetVertexBuffer();
            int textureBuffer = VertexBuffer.GetInstance().GetTextureBuffer();

            // Create colored rectangle
            ColorShaderProgram colorProgram = new ColorShaderProgram(
                "./Shaders/vColorShader.glsl", "./Shaders/fColorShader.glsl",
                vertexBuffer);
            _colorRect = new Renderable(colorProgram);

            // Create textured rectangle
            TextureShaderProgram textureProgram = new TextureShaderProgram(
                "./Shaders/vTextureShader.glsl", "./Shaders/fTextureShader.glsl",
                vertexBuffer, textureBuffer);
            _tank = new TextureRenderable(textureProgram, _tankTextureName);
            _bonusStar = new TextureRenderable(textureProgram, _bonusStarTextureName);
            _bonusTank = new TextureRenderable(textureProgram, _bonusTankTextureName);

            int halfOfField = _fieldWidth / 2;
            _camera = new Camera(
                new Vector2(halfOfField, halfOfField),  // Position of the camera
                _fieldWidth,                            // Width of field
                new Vector4(10, 10, 380, 380));         // viewport (orgX, orgY, width, height)

            Textures.GetInstance().LoadTexture(_tankTextureName);
            Textures.GetInstance().LoadTexture(_bonusStarTextureName);
            Textures.GetInstance().LoadTexture(_bonusTankTextureName);
        }

        protected override void OnRenderFrame(FrameEventArgs e)
        {
            base.OnRenderFrame(e);

            GL.ClearColor(Color4.SaddleBrown);
            GL.Clear(ClearBufferMask.ColorBufferBit);

            _camera.SetupViewProjection();

            DrawColorRect(1, 1, Color4.LightGreen, 1f, 5f);
            DrawColorRect(5, 7, Color4.Linen, 2f, 2f);
            DrawTank(_x, _y);
            DrawBonusStar(7, 1);
            DrawBonusTank(5, 5);

            SwapBuffers();
        }

        protected override void OnUpdateFrame(FrameEventArgs e)
        {
            base.OnUpdateFrame(e);
            Input.GetInstance().Update();

            if (Input.GetInstance().IsKeyPressed((int)Engine.Key.W) ||
                Input.GetInstance().IsKeyPressed((int)Engine.Key.Up))
            {
                _y += _speed;
            }

            if (Input.GetInstance().IsKeyPressed((int)Engine.Key.A) ||
                Input.GetInstance().IsKeyPressed((int)Engine.Key.Left))
            {
                _x -= _speed;
            }

            if (Input.GetInstance().IsKeyPressed((int)Engine.Key.S) ||
                Input.GetInstance().IsKeyPressed((int)Engine.Key.Down))
            {
                _y -= _speed;
            }

            if (Input.GetInstance().IsKeyPressed((int)Engine.Key.D) ||
                Input.GetInstance().IsKeyPressed((int)Engine.Key.Right))
            {
                _x += _speed;
            }
        }

        protected override void OnKeyDown(KeyboardKeyEventArgs e)
        {
            base.OnKeyDown(e);
            Input.GetInstance().OnKeyDown((int)e.Key);
        }

        protected override void OnKeyUp(KeyboardKeyEventArgs e)
        {
            base.OnKeyUp(e);
            Input.GetInstance().OnKeyUp((int)e.Key);
        }

        private void DrawColorRect(
            float x, float y, Color4 color,
            float width = 1f, float height = 1f)
        {
            _colorRect.GetXform().SetPosition(x, y);
            _colorRect.SetColor(color);
            _colorRect.GetXform().SetSize(width, height);
            _colorRect.Draw(_camera.GetVPMatrix());
        }

        private void DrawTank(float x, float y)
        {
            _tank.GetXform().SetPosition(x, y);
            _tank.Draw(_camera.GetVPMatrix());
        }

        private void DrawBonusStar(float x, float y)
        {
            _bonusStar.GetXform().SetPosition(x, y);
            _bonusStar.Draw(_camera.GetVPMatrix());
        }

        private void DrawBonusTank(float x, float y)
        {
            _bonusTank.GetXform().SetPosition(x, y);
            _bonusTank.Draw(_camera.GetVPMatrix());
        }
    }
}
